@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * dhcpcd - DHCP client daemon
- * Copyright (c) 2006-2020 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2021 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -26,7 +26,7 @@
  * SUCH DAMAGE.
  */
 
-static const char dhcpcd_copyright[] = "Copyright (c) 2006-2020 Roy Marples";
+static const char dhcpcd_copyright[] = "Copyright (c) 2006-2021 Roy Marples";
 
 #include <sys/file.h>
 #include <sys/ioctl.h>
@@ -107,7 +107,7 @@ printf("usage: "PACKAGE"\t[-146ABbDdEGgHJKLMNPpqTV]\n"
 	"\t\t[-C, --nohook hook] [-c, --script script]\n"
 	"\t\t[-e, --env value] [-F, --fqdn FQDN] [-f, --config file]\n"
 	"\t\t[-h, --hostname hostname] [-I, --clientid clientid]\n"
-	"\t\t[-i, --vendorclassid vendorclassid] [-j, --logfile logfile]\n" 
+	"\t\t[-i, --vendorclassid vendorclassid] [-j, --logfile logfile]\n"
 	"\t\t[-l, --leasetime seconds] [-m, --metric metric]\n"
 	"\t\t[-O, --nooption option] [-o, --option option]\n"
 	"\t\t[-Q, --require option] [-r, --request address]\n"
@@ -196,7 +196,7 @@ handle_exit_timeout(void *arg)
 
 	ctx = arg;
 	logerrx("timed out");
-	if (!(ctx->options & DHCPCD_MASTER)) {
+	if (!(ctx->options & DHCPCD_MANAGER)) {
 		struct interface *ifp;
 
 		TAILQ_FOREACH(ifp, ctx->ifaces, next) {
@@ -370,12 +370,12 @@ dhcpcd_daemonise(struct dhcpcd_ctx *ctx)
 
 	/*
 	 * Stop writing to stderr.
-	 * On the happy path, only the master process writes to stderr,
+	 * On the happy path, only the manager process writes to stderr,
 	 * so this just stops wasting fprintf calls to nowhere.
 	 * All other calls - ie errors in privsep processes or script output,
 	 * will error when printing.
 	 * If we *really* want to fix that, then we need to suck
-	 * stderr/stdout in the master process and either disacrd it or pass
+	 * stderr/stdout in the manager process and either disacrd it or pass
 	 * it to the launcher process and then to stderr.
 	 */
 	logopts &= ~LOGERR_ERR;
@@ -427,7 +427,7 @@ stop_interface(struct interface *ifp, const char *reason)
 	ifp->active = IF_INACTIVE;
 	ifp->options->options &= ~DHCPCD_STOPPING;
 
-	if (!(ctx->options & (DHCPCD_MASTER | DHCPCD_TEST)))
+	if (!(ctx->options & (DHCPCD_MANAGER | DHCPCD_TEST)))
 		eloop_exit(ctx->eloop, EXIT_FAILURE);
 }
 
@@ -980,7 +980,7 @@ dhcpcd_prestartinterface(void *arg)
 	} else
 		randmac_down = false;
 
-	if ((!(ctx->options & DHCPCD_MASTER) ||
+	if ((!(ctx->options & DHCPCD_MANAGER) ||
 	    ifp->options->options & DHCPCD_IF_UP || randmac_down) &&
 	    !(ifp->flags & IFF_UP))
 	{
@@ -1105,6 +1105,7 @@ out:
 		if_free(ifp);
 	}
 	free(ifs);
+	if_freeifaddrs(ctx, &ifaddrs);
 
 	return e;
 }
@@ -1241,6 +1242,7 @@ dhcpcd_linkoverflow(struct dhcpcd_ctx *ctx)
 	if_markaddrsstale(ctx->ifaces);
 	if_learnaddrs(ctx, ctx->ifaces, &ifaddrs);
 	if_deletestaleaddrs(ctx->ifaces);
+	if_freeifaddrs(ctx, &ifaddrs);
 }
 
 void
@@ -1311,8 +1313,8 @@ reload_config(struct dhcpcd_ctx *ctx)
 	/* We need to preserve these options. */
 	if (ctx->options & DHCPCD_STARTED)
 		ifo->options |= DHCPCD_STARTED;
-	if (ctx->options & DHCPCD_MASTER)
-		ifo->options |= DHCPCD_MASTER;
+	if (ctx->options & DHCPCD_MANAGER)
+		ifo->options |= DHCPCD_MANAGER;
 	if (ctx->options & DHCPCD_DAEMONISED)
 		ifo->options |= DHCPCD_DAEMONISED;
 	if (ctx->options & DHCPCD_PRIVSEP)
@@ -1940,12 +1942,6 @@ main(int argc, char **argv, char **envp)
 			sig = SIGHUP;
 			siga = "HUP";
 			break;
-		case 'g':
-		case 'p':
-			/* Force going via command socket as we're
-			 * out of user definable signals. */
-			i = 4;
-			break;
 		case 'q':
 			/* -qq disables console output entirely.
 			 * This is important for systemd because it logs
@@ -1988,7 +1984,7 @@ main(int argc, char **argv, char **envp)
 	}
 
 	if (optind != argc - 1)
-		ctx.options |= DHCPCD_MASTER;
+		ctx.options |= DHCPCD_MANAGER;
 
 	logsetopts(logopts);
 	logopen(ctx.logfile);
@@ -2068,7 +2064,7 @@ main(int argc, char **argv, char **envp)
 printpidfile:
 		/* If we have any other args, we should run as a single dhcpcd
 		 *  instance for that interface. */
-		if (optind == argc - 1 && !(ctx.options & DHCPCD_MASTER)) {
+		if (optind == argc - 1 && !(ctx.options & DHCPCD_MANAGER)) {
 			const char *per;
 			const char *ifname;
 
@@ -2094,7 +2090,15 @@ printpidfile:
 		} else {
 			snprintf(ctx.pidfile, sizeof(ctx.pidfile),
 			    PIDFILE, "", "", "");
-			ctx.options |= DHCPCD_MASTER;
+			ctx.options |= DHCPCD_MANAGER;
+
+			/*
+			 * If we are given any interfaces, we
+			 * cannot send a signal as that would impact
+			 * other interfaces.
+			 */
+			if (optind != argc)
+				sig = 0;
 		}
 		if (ctx.options & DHCPCD_PRINT_PIDFILE) {
 			printf("%s\n", ctx.pidfile);
@@ -2130,15 +2134,13 @@ printpidfile:
 		if (pid != 0 && pid != -1)
 			loginfox("sending signal %s to pid %d", siga, pid);
 		if (pid == 0 || pid == -1 || kill(pid, sig) != 0) {
-			if (sig != SIGHUP && sig != SIGUSR1 && errno != EPERM)
-				logerrx(PACKAGE" not running");
 			if (pid != 0 && pid != -1 && errno != ESRCH) {
 				logerr("kill");
 				goto exit_failure;
 			}
 			unlink(ctx.pidfile);
-			if (sig != SIGHUP && sig != SIGUSR1)
-				goto exit_failure;
+			/* We can still continue and send the command
+			 * via the control socket. */
 		} else {
 			struct timespec ts;
 
@@ -2170,7 +2172,7 @@ printpidfile:
 	{
 		ctx.options |= DHCPCD_FORKED; /* pretend child process */
 #ifdef PRIVSEP
-		if (IN_PRIVSEP(&ctx) && ps_mastersandbox(&ctx, NULL) == -1)
+		if (IN_PRIVSEP(&ctx) && ps_managersandbox(&ctx, NULL) == -1)
 			goto exit_failure;
 #endif
 		ifp = calloc(1, sizeof(*ifp));
@@ -2207,17 +2209,13 @@ printpidfile:
 	}
 #endif
 
-	/* Test against siga instead of sig to avoid gcc
-	 * warning about a bogus potential signed overflow.
-	 * The end result will be the same. */
-	if ((siga == NULL || i == 4 || ctx.ifc != 0) &&
-	    !(ctx.options & DHCPCD_TEST))
-	{
+	/* Try and contact the manager process to send the instruction. */
+	if (!(ctx.options & DHCPCD_TEST)) {
 		ctx.options |= DHCPCD_FORKED; /* avoid socket unlink */
-		if (!(ctx.options & DHCPCD_MASTER))
+		if (!(ctx.options & DHCPCD_MANAGER))
 			ctx.control_fd = control_open(argv[optind], family,
 			    ctx.options & DHCPCD_DUMPLEASE);
-		if (!(ctx.options & DHCPCD_MASTER) && ctx.control_fd == -1)
+		if (!(ctx.options & DHCPCD_MANAGER) && ctx.control_fd == -1)
 			ctx.control_fd = control_open(argv[optind], AF_UNSPEC,
 			    ctx.options & DHCPCD_DUMPLEASE);
 		if (ctx.control_fd == -1)
@@ -2226,7 +2224,7 @@ printpidfile:
 		if (ctx.control_fd != -1) {
 #ifdef PRIVSEP
 			if (IN_PRIVSEP(&ctx) &&
-			    ps_mastersandbox(&ctx, NULL) == -1)
+			    ps_managersandbox(&ctx, NULL) == -1)
 				goto exit_failure;
 #endif
 			if (!(ctx.options & DHCPCD_DUMPLEASE))
@@ -2249,9 +2247,14 @@ printpidfile:
 		} else {
 			if (errno != ENOENT)
 				logerr("%s: control_open", __func__);
-			if (ctx.options & DHCPCD_DUMPLEASE) {
+			/* If asking dhcpcd to exit and we failed to
+			 * send a signal or a message then we
+			 * don't proceed past here. */
+			if (ctx.options & DHCPCD_DUMPLEASE ||
+			    sig == SIGTERM || sig == SIGALRM)
+			{
 				if (errno == ENOENT)
-					logerrx("dhcpcd is not running");
+					logerrx(PACKAGE" is not running");
 				goto exit_failure;
 			}
 			if (errno == EPERM || errno == EACCES)
@@ -2284,7 +2287,7 @@ printpidfile:
 
 #if defined(USE_SIGNALS) && !defined(THERE_IS_NO_FORK)
 	if (!(ctx.options & DHCPCD_DAEMONISE))
-		goto start_master;
+		goto start_manager;
 
 	if (xsocketpair(AF_UNIX, SOCK_DGRAM | SOCK_CXNB, 0, fork_fd) == -1 ||
 	    (ctx.stderr_valid &&
@@ -2368,7 +2371,7 @@ printpidfile:
 			    dhcpcd_stderr_cb, &ctx);
 		}
 #ifdef PRIVSEP
-		if (IN_PRIVSEP(&ctx) && ps_mastersandbox(&ctx, NULL) == -1)
+		if (IN_PRIVSEP(&ctx) && ps_managersandbox(&ctx, NULL) == -1)
 			goto exit_failure;
 #endif
 		goto run_loop;
@@ -2376,8 +2379,8 @@ printpidfile:
 
 	/* We have now forked, setsid, forked once more.
 	 * From this point on, we are the controlling daemon. */
-	logdebugx("spawned master process on PID %d", getpid());
-start_master:
+	logdebugx("spawned manager process on PID %d", getpid());
+start_manager:
 	ctx.options |= DHCPCD_STARTED;
 	if ((pid = pidfile_lock(ctx.pidfile)) != 0) {
 		logerr("%s: pidfile_lock %d", __func__, pid);
@@ -2408,7 +2411,7 @@ start_master:
 
 	if (!(ctx.options & DHCPCD_TEST)) {
 		if (control_start(&ctx,
-		    ctx.options & DHCPCD_MASTER ?
+		    ctx.options & DHCPCD_MANAGER ?
 		    NULL : argv[optind], family) == -1)
 		{
 			logerr("%s: control_start", __func__);
@@ -2420,13 +2423,13 @@ start_master:
 	/* Start any dev listening plugin which may want to
 	 * change the interface name provided by the kernel */
 	if (!IN_PRIVSEP(&ctx) &&
-	    (ctx.options & (DHCPCD_MASTER | DHCPCD_DEV)) ==
-	    (DHCPCD_MASTER | DHCPCD_DEV))
+	    (ctx.options & (DHCPCD_MANAGER | DHCPCD_DEV)) ==
+	    (DHCPCD_MANAGER | DHCPCD_DEV))
 		dev_start(&ctx, dhcpcd_handleinterface);
 #endif
 
 	setproctitle("%s%s%s",
-	    ctx.options & DHCPCD_MASTER ? "[master]" : argv[optind],
+	    ctx.options & DHCPCD_MANAGER ? "[manager]" : argv[optind],
 	    ctx.options & DHCPCD_IPV4 ? " [ip4]" : "",
 	    ctx.options & DHCPCD_IPV6 ? " [ip6]" : "");
 
@@ -2450,7 +2453,7 @@ start_master:
 	eloop_event_add(ctx.eloop, ctx.link_fd, dhcpcd_handlelink, &ctx);
 
 #ifdef PRIVSEP
-	if (IN_PRIVSEP(&ctx) && ps_mastersandbox(&ctx, "stdio route") == -1)
+	if (IN_PRIVSEP(&ctx) && ps_managersandbox(&ctx, "stdio route") == -1)
 		goto exit_failure;
 #endif
 
@@ -2497,6 +2500,8 @@ start_master:
 			dhcpcd_initstate1(ifp, argc, argv, 0);
 	}
 	if_learnaddrs(&ctx, ctx.ifaces, &ifaddrs);
+	if_freeifaddrs(&ctx, &ifaddrs);
+	ifaddrs = NULL;
 
 	if (ctx.options & DHCPCD_BACKGROUND)
 		dhcpcd_daemonise(&ctx);
@@ -2511,7 +2516,7 @@ start_master:
 	}
 
 	if (!(ctx.options & DHCPCD_BACKGROUND)) {
-		if (ctx.options & DHCPCD_MASTER)
+		if (ctx.options & DHCPCD_MANAGER)
 			t = ifo->timeout;
 		else {
 			t = 0;
@@ -2567,14 +2572,9 @@ exit_failure:
 exit1:
 	if (!(ctx.options & DHCPCD_TEST) && control_stop(&ctx) == -1)
 		logerr("%s: control_stop", __func__);
-	if (ifaddrs != NULL) {
-#ifdef PRIVSEP_GETIFADDRS
-		if (IN_PRIVSEP(&ctx))
-			free(ifaddrs);
-		else
-#endif
-			freeifaddrs(ifaddrs);
-	}
+
+	if_freeifaddrs(&ctx, &ifaddrs);
+
 	/* ps_stop will clear DHCPCD_PRIVSEP but we need to
 	 * remember it to avoid attemping to remove the pidfile */
 	oi = ctx.options & DHCPCD_PRIVSEP ? 1 : 0;
