@@ -549,15 +549,15 @@ if_getnetlink(struct dhcpcd_ctx *ctx, struct iovec *iov, int fd, int flags,
 	    .msg_name = &nladdr, .msg_namelen = sizeof(nladdr),
 	    .msg_iov = iov, .msg_iovlen = 1,
 	};
-	ssize_t len;
+	size_t len;
 	struct nlmsghdr *nlm;
 	int r = 0;
 	unsigned int again;
 	bool terminated;
 
 recv_again:
-	len = recvmsg(fd, &msg, flags);
-	if (len == -1 || len == 0)
+	len = (size_t)recvmsg(fd, &msg, flags);
+	if (len == 0 || (ssize_t)len == -1)
 		return (int)len;
 
 	/* Check sender */
@@ -573,7 +573,7 @@ recv_again:
 	again = 0;
 	terminated = false;
 	for (nlm = iov->iov_base;
-	     nlm && NLMSG_OK(nlm, (size_t)len);
+	     nlm && NLMSG_OK(nlm, len);
 	     nlm = NLMSG_NEXT(nlm, len))
 	{
 		again = (nlm->nlmsg_flags & NLM_F_MULTI);
@@ -1138,7 +1138,8 @@ if_sendnetlink(struct dhcpcd_ctx *ctx, int protocol, struct nlmsghdr *hdr,
 }
 
 #define NLMSG_TAIL(nmsg)						\
-	((struct rtattr *)(((ptrdiff_t)(nmsg))+NLMSG_ALIGN((nmsg)->nlmsg_len)))
+	((struct rtattr *)(void *)(((char *)(nmsg)) + \
+	    NLMSG_ALIGN((nmsg)->nlmsg_len)))
 
 static int
 add_attr_l(struct nlmsghdr *n, unsigned short maxlen, unsigned short type,
@@ -2106,7 +2107,12 @@ if_setup_inet6(const struct interface *ifp)
 
 	snprintf(path, sizeof(path), "%s/%s/autoconf", p_conf, ifp->name);
 	ra = check_proc_int(ctx, path);
-	if (ra != 1 && ra != -1) {
+	if (ra == -1) {
+		/* The sysctl probably doesn't exist, but this isn't an
+		 * error as such so just log it and continue */
+		if (errno != ENOENT)
+			logerr("%s: %s", __func__, path);
+	} else if (ra != 0) {
 		if (if_writepathuint(ctx, path, 0) == -1)
 			logerr("%s: %s", __func__, path);
 	}
