@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: BSD-2-Clause */
 /*
- * Privilege Separation for dhcpcd, privileged actioneer
- * Copyright (c) 2006-2020 Roy Marples <roy@marples.name>
+ * Privilege Separation for dhcpcd, privileged proxy
+ * Copyright (c) 2006-2021 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -200,12 +200,26 @@ ps_root_writeerror(struct dhcpcd_ctx *ctx, ssize_t result,
 		{ .iov_base = &psr, .iov_len = sizeof(psr) },
 		{ .iov_base = data, .iov_len = len },
 	};
+	ssize_t err;
 
 #ifdef PRIVSEP_DEBUG
 	logdebugx("%s: result %zd errno %d", __func__, result, errno);
 #endif
 
-	return writev(ctx->ps_root_fd, iov, __arraycount(iov));
+	err = writev(ctx->ps_root_fd, iov, __arraycount(iov));
+
+	/* Error sending the message? Try sending the error of sending. */
+	if (err == -1) {
+		logerr("%s: result=%zd, data=%p, len=%zu",
+		    __func__, result, data, len);
+		psr.psr_result = err;
+		psr.psr_errno = errno;
+		iov[1].iov_base = NULL;
+		iov[1].iov_len = 0;
+		err = writev(ctx->ps_root_fd, iov, __arraycount(iov));
+	}
+
+	return err;
 }
 
 static ssize_t
@@ -593,7 +607,7 @@ ps_root_recvmsgcb(void *arg, struct ps_msghdr *psm, struct msghdr *msg)
 		break;
 #endif
 	default:
-		err = ps_root_os(psm, msg, &rdata, &rlen);
+		err = ps_root_os(psm, msg, &rdata, &rlen, &free_rdata);
 		break;
 	}
 
@@ -641,10 +655,10 @@ ps_root_startcb(void *arg)
 {
 	struct dhcpcd_ctx *ctx = arg;
 
-	if (ctx->options & DHCPCD_MASTER)
-		setproctitle("[privileged actioneer]");
+	if (ctx->options & DHCPCD_MANAGER)
+		setproctitle("[privileged proxy]");
 	else
-		setproctitle("[privileged actioneer] %s%s%s",
+		setproctitle("[privileged proxy] %s%s%s",
 		    ctx->ifv[0],
 		    ctx->options & DHCPCD_IPV4 ? " [ip4]" : "",
 		    ctx->options & DHCPCD_IPV6 ? " [ip6]" : "");
@@ -653,7 +667,7 @@ ps_root_startcb(void *arg)
 
 	/* Open network sockets for sending.
 	 * This is a small bit wasteful for non sandboxed OS's
-	 * but makes life very easy for unicasting DHCPv6 in non master
+	 * but makes life very easy for unicasting DHCPv6 in non manager
 	 * mode as we no longer care about address selection.
 	 * We can't call shutdown SHUT_RD on the socket because it's
 	 * not connectd. All we can do is try and set a zero sized
@@ -700,8 +714,8 @@ ps_root_startcb(void *arg)
 #ifdef PLUGIN_DEV
 	/* Start any dev listening plugin which may want to
 	 * change the interface name provided by the kernel */
-	if ((ctx->options & (DHCPCD_MASTER | DHCPCD_DEV)) ==
-	    (DHCPCD_MASTER | DHCPCD_DEV))
+	if ((ctx->options & (DHCPCD_MANAGER | DHCPCD_DEV)) ==
+	    (DHCPCD_MANAGER | DHCPCD_DEV))
 		dev_start(ctx, ps_root_handleinterface);
 #endif
 
@@ -981,17 +995,17 @@ ps_root_getifaddrs(struct dhcpcd_ctx *ctx, struct ifaddrs **ifahead)
 		len -= ALIGN(sizeof(*ifa)) +
 		    ALIGN(IFNAMSIZ) + ALIGN(sizeof(salen) * IFA_NADDRS);
 
-#define	COPYOUTSA(addr)						\
-	do {							\
-		memcpy(&salen, sap, sizeof(salen));		\
-		if (len < salen)				\
-			goto err;				\
-		if (salen != 0) {				\
-			(addr) = (struct sockaddr *)bp;		\
-			bp += ALIGN(salen);			\
-			len -= ALIGN(salen);			\
-		}						\
-		sap += sizeof(salen);				\
+#define	COPYOUTSA(addr)							\
+	do {								\
+		memcpy(&salen, sap, sizeof(salen));			\
+		if (len < salen)					\
+			goto err;					\
+		if (salen != 0) {					\
+			(addr) = (struct sockaddr *)(void *)bp;		\
+			bp += ALIGN(salen);				\
+			len -= ALIGN(salen);				\
+		}							\
+		sap += sizeof(salen);					\
 	} while (0 /* CONSTCOND */)
 
 		COPYOUTSA(ifa->ifa_addr);
