@@ -1,7 +1,7 @@
 /* SPDX-License-Identifier: BSD-2-Clause */
 /*
  * Privilege Separation for dhcpcd
- * Copyright (c) 2006-2020 Roy Marples <roy@marples.name>
+ * Copyright (c) 2006-2021 Roy Marples <roy@marples.name>
  * All rights reserved
 
  * Redistribution and use in source and binary forms, with or without
@@ -33,8 +33,8 @@
  * or address specific listener.
  * Spawn an unpriv process to send/receive common network data.
  * Then drop all privs and start running.
- * Every process aside from the privileged actioneer is chrooted.
- * All privsep processes ignore signals - only the master process accepts them.
+ * Every process aside from the privileged proxy is chrooted.
+ * All privsep processes ignore signals - only the manager process accepts them.
  *
  * dhcpcd will maintain the config file in the chroot, no need to handle
  * this in a script or something.
@@ -81,6 +81,15 @@
 #include <util.h>
 #endif
 
+/* CMSG_ALIGN is a Linux extension */
+#ifndef CMSG_ALIGN
+#define CMSG_ALIGN(n)	(CMSG_SPACE((n)) - CMSG_SPACE(0))
+#endif
+
+/* Calculate number of padding bytes to achieve 'struct cmsghdr' alignment */
+#define CALC_CMSG_PADLEN(has_cmsg, pos) \
+    ((has_cmsg) ? (socklen_t)(CMSG_ALIGN((pos)) - (pos)) : 0)
+
 int
 ps_init(struct dhcpcd_ctx *ctx)
 {
@@ -135,33 +144,27 @@ ps_dropprivs(struct dhcpcd_ctx *ctx)
 
 	struct rlimit rzero = { .rlim_cur = 0, .rlim_max = 0 };
 
+	/* Prohibit new files, sockets, etc */
+	/*
+	 * If poll(2) is called with nfds>RLIMIT_NOFILE then it returns EINVAL.
+	 * We don't know the final value of nfds at this point *easily*.
+	 * Sadly, this is a POSIX limitation and most platforms adhere to it.
+	 * However, some are not that strict and are whitelisted below.
+	 * Also, if we're not using poll then we can be restrictive.
+	 *
+	 * For the non whitelisted platforms there should be a sandbox to
+	 * fallback to where we don't allow new files, etc:
+	 *      Linux:seccomp, FreeBSD:capsicum, OpenBSD:pledge
+	 * Solaris users are sadly out of luck on both counts.
+	 */
+#if defined(__NetBSD__) || defined(__DragonFly__) || \
+    defined(HAVE_KQUEUE) || defined(HAVE_EPOLL)
+	/* The control proxy *does* need to create new fd's via accept(2). */
 	if (ctx->ps_control_pid != getpid()) {
-		/* Prohibit new files, sockets, etc */
-#if defined(__linux__) || defined(__sun) || defined(__OpenBSD__)
-		/*
-		 * If poll(2) is called with nfds > RLIMIT_NOFILE
-		 * then it returns EINVAL.
-		 * This blows.
-		 * Do the best we can and limit to what we need.
-		 * An attacker could potentially close a file and
-		 * open a new one still, but that cannot be helped.
-		 */
-		unsigned long maxfd;
-		maxfd = (unsigned long)eloop_event_count(ctx->eloop);
-		if (IN_PRIVSEP_SE(ctx))
-			maxfd++; /* XXX why? */
-
-		struct rlimit rmaxfd = {
-		    .rlim_cur = maxfd,
-		    .rlim_max = maxfd
-		};
-		if (setrlimit(RLIMIT_NOFILE, &rmaxfd) == -1)
-			logerr("setrlimit RLIMIT_NOFILE");
-#else
 		if (setrlimit(RLIMIT_NOFILE, &rzero) == -1)
 			logerr("setrlimit RLIMIT_NOFILE");
-#endif
 	}
+#endif
 
 #define DHC_NOCHKIO	(DHCPCD_STARTED | DHCPCD_DAEMONISE)
 	/* Prohibit writing to files.
@@ -363,13 +366,28 @@ ps_dostart(struct dhcpcd_ctx *ctx,
 		return pid;
 	}
 
+
+#ifdef PLUGIN_DEV
+	/* If we are not the root process, stop listening to devices. */
+	if (priv_fd != &ctx->ps_root_fd)
+		dev_stop(ctx);
+#endif
+
 	ctx->options |= DHCPCD_FORKED;
 	if (ctx->fork_fd != -1) {
 		close(ctx->fork_fd);
 		ctx->fork_fd = -1;
 	}
 	pidfile_clean();
+
 	eloop_clear(ctx->eloop);
+	eloop_signal_set_cb(ctx->eloop,
+	    dhcpcd_signals, dhcpcd_signals_len, signal_cb, ctx);
+	/* ctx->sigset aready has the initial sigmask set in main() */
+	if (eloop_signal_mask(ctx->eloop, NULL) == -1) {
+		logerr("%s: eloop_signal_mask", __func__);
+		goto errexit;
+	}
 
 	/* We are not root */
 	if (priv_fd != &ctx->ps_root_fd) {
@@ -393,15 +411,6 @@ ps_dostart(struct dhcpcd_ctx *ctx,
 		ctx->ps_inet_fd = -1;
 	}
 
-	eloop_signal_set_cb(ctx->eloop,
-	    dhcpcd_signals, dhcpcd_signals_len, signal_cb, ctx);
-
-	/* ctx->sigset aready has the initial sigmask set in main() */
-	if (eloop_signal_mask(ctx->eloop, NULL) == -1) {
-		logerr("%s: eloop_signal_mask", __func__);
-		goto errexit;
-	}
-
 	if (eloop_event_add(ctx->eloop, *priv_fd, recv_msg, recv_ctx) == -1)
 	{
 		logerr("%s: eloop_event_add", __func__);
@@ -466,7 +475,7 @@ ps_start(struct dhcpcd_ctx *ctx)
 	case 0:
 		return 0;
 	default:
-		logdebugx("spawned privileged actioneer on PID %d", pid);
+		logdebugx("spawned privileged proxy on PID %d", pid);
 	}
 
 	/* No point in spawning the generic network listener if we're
@@ -532,7 +541,7 @@ ps_entersandbox(const char *_pledge, const char **sandbox)
 }
 
 int
-ps_mastersandbox(struct dhcpcd_ctx *ctx, const char *_pledge)
+ps_managersandbox(struct dhcpcd_ctx *ctx, const char *_pledge)
 {
 	const char *sandbox = NULL;
 	bool forked;
@@ -583,7 +592,7 @@ ps_mastersandbox(struct dhcpcd_ctx *ctx, const char *_pledge)
 		return -1;
 	} else if (ctx->options & DHCPCD_LAUNCHER ||
 		  ((!(ctx->options & DHCPCD_DAEMONISE)) &&
-		   ctx->options & DHCPCD_MASTER))
+		   ctx->options & DHCPCD_MANAGER))
 		logdebugx("sandbox: %s", sandbox);
 	return 0;
 }
@@ -607,7 +616,7 @@ ps_stop(struct dhcpcd_ctx *ctx)
 		ret = r;
 
 	/* We've been chrooted, so we need to tell the
-	 * privileged actioneer to remove the pidfile. */
+	 * privileged proxy to remove the pidfile. */
 	ps_root_unlink(ctx, ctx->pidfile);
 
 	r = ps_root_stop(ctx);
@@ -656,9 +665,11 @@ ps_unrollmsg(struct msghdr *msg, struct ps_msghdr *psm,
     const void *data, size_t len)
 {
 	uint8_t *datap, *namep, *controlp;
+	socklen_t cmsg_padlen =
+	    CALC_CMSG_PADLEN(psm->ps_controllen, psm->ps_namelen);
 
 	namep = UNCONST(data);
-	controlp = namep + psm->ps_namelen;
+	controlp = namep + psm->ps_namelen + cmsg_padlen;
 	datap = controlp + psm->ps_controllen;
 
 	if (psm->ps_namelen != 0) {
@@ -678,7 +689,7 @@ ps_unrollmsg(struct msghdr *msg, struct ps_msghdr *psm,
 			return -1;
 		}
 		msg->msg_control = controlp;
-		len -= psm->ps_controllen;
+		len -= psm->ps_controllen + cmsg_padlen;
 	} else
 		msg->msg_control = NULL;
 	msg->msg_controllen = psm->ps_controllen;
@@ -699,9 +710,11 @@ ssize_t
 ps_sendpsmmsg(struct dhcpcd_ctx *ctx, int fd,
     struct ps_msghdr *psm, const struct msghdr *msg)
 {
+	long padding[1] = { 0 };
 	struct iovec iov[] = {
 		{ .iov_base = UNCONST(psm), .iov_len = sizeof(*psm) },
 		{ .iov_base = NULL, },	/* name */
+		{ .iov_base = NULL, },	/* control padding */
 		{ .iov_base = NULL, },	/* control */
 		{ .iov_base = NULL, },	/* payload 1 */
 		{ .iov_base = NULL, },	/* payload 2 */
@@ -713,6 +726,7 @@ ps_sendpsmmsg(struct dhcpcd_ctx *ctx, int fd,
 	if (msg != NULL) {
 		struct iovec *iovp = &iov[1];
 		int i;
+		socklen_t cmsg_padlen;
 
 		psm->ps_namelen = msg->msg_namelen;
 		psm->ps_controllen = (socklen_t)msg->msg_controllen;
@@ -720,9 +734,17 @@ ps_sendpsmmsg(struct dhcpcd_ctx *ctx, int fd,
 		iovp->iov_base = msg->msg_name;
 		iovp->iov_len = msg->msg_namelen;
 		iovp++;
+
+		cmsg_padlen =
+		    CALC_CMSG_PADLEN(msg->msg_controllen, msg->msg_namelen);
+		assert(cmsg_padlen <= sizeof(padding));
+		iovp->iov_len = cmsg_padlen;
+		iovp->iov_base = cmsg_padlen != 0 ? padding : NULL;
+		iovp++;
+
 		iovp->iov_base = msg->msg_control;
 		iovp->iov_len = msg->msg_controllen;
-		iovlen = 3;
+		iovlen = 4;
 
 		for (i = 0; i < (int)msg->msg_iovlen; i++) {
 			if ((size_t)(iovlen + i) > __arraycount(iov)) {
@@ -833,6 +855,8 @@ ps_sendcmdmsg(int fd, uint16_t cmd, const struct msghdr *msg)
 		{ .iov_base = data, .iov_len = 0 },
 	};
 	size_t dl = sizeof(data);
+	socklen_t cmsg_padlen =
+	    CALC_CMSG_PADLEN(msg->msg_controllen, msg->msg_namelen);
 
 	if (msg->msg_namelen != 0) {
 		if (msg->msg_namelen > dl)
@@ -844,8 +868,13 @@ ps_sendcmdmsg(int fd, uint16_t cmd, const struct msghdr *msg)
 	}
 
 	if (msg->msg_controllen != 0) {
-		if (msg->msg_controllen > dl)
+		if (msg->msg_controllen + cmsg_padlen > dl)
 			goto nobufs;
+		if (cmsg_padlen != 0) {
+			memset(p, 0, cmsg_padlen);
+			p += cmsg_padlen;
+			dl -= cmsg_padlen;
+		}
 		psm.ps_controllen = (socklen_t)msg->msg_controllen;
 		memcpy(p, msg->msg_control, msg->msg_controllen);
 		p += msg->msg_controllen;
@@ -856,7 +885,8 @@ ps_sendcmdmsg(int fd, uint16_t cmd, const struct msghdr *msg)
 	if (psm.ps_datalen > dl)
 		goto nobufs;
 
-	iov[1].iov_len = psm.ps_namelen + psm.ps_controllen + psm.ps_datalen;
+	iov[1].iov_len =
+	    psm.ps_namelen + psm.ps_controllen + psm.ps_datalen + cmsg_padlen;
 	if (psm.ps_datalen != 0)
 		memcpy(p, msg->msg_iov[0].iov_base, psm.ps_datalen);
 	return writev(fd, iov, __arraycount(iov));
@@ -941,9 +971,6 @@ ps_recvpsmsg(struct dhcpcd_ctx *ctx, int fd,
 		logdebugx("process %d stopping", getpid());
 #endif
 		ps_free(ctx);
-#ifdef PLUGIN_DEV
-		dev_stop(ctx);
-#endif
 		eloop_exit(ctx->eloop, len != -1 ? EXIT_SUCCESS : EXIT_FAILURE);
 		return len;
 	}
